//=======================================================================================================																					
//  						'Tiny 13A led sensor AirWick' by Alex Shelehov (C)
//												v.1.1
//=======================================================================================================

//  
#define F_CPU 1200000UL

// 
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdbool.h>
#include <avr/pgmspace.h>

// 
#define PIN_MOTOR 			PB0	
#define PIN_BUTTON_MOTOR 	PB1
#define PIN_BUTTON_MODE 	PB2
#define PIN_LED_N 			PB3
#define PIN_LED_P 			PB4

//  wdt n=9.6 (     n)

#define MOTOR_WORK_TIME 	500						//   , ,  !
#define MAX_LIGHT_TIME		375						//     ,  ,     (1  = 375)
#define MIN_LAST_TIME		94						//     "",     , 15 = 94
#define DEFAULT_MODE		0						//    ,   EEPROM  (  )
#define MODE_COUNT			4						//   

uint8_t Addr EEMEM;									//    EEPROM   "Addr"

uint16_t main_timer = 0; 							//  () 

const uint16_t motor_on_time[MODE_COUNT] PROGMEM = 
	{ 375, 375, 1125, 1125}; 						// 1  = 375, 3  = 1125.      (,  )

const uint8_t min_light_time[MODE_COUNT] PROGMEM = 
	{ 16, 31, 16, 31};								// 2.5  = 16, 5  = 31.    ,  ,  " "

volatile bool sleep_flag = false;
volatile bool button_motor_flag = false;
volatile bool button_mode_flag = false;

//---------------------------------------------------------------------------------------------------
//										  
//---------------------------------------------------------------------------------------------------

//  
uint16_t readLED(void){
	uint16_t j;
	PORTB |= _BV(PIN_LED_N);						//     
	_delay_ms(1);									//     
	DDRB  &= ~_BV(PIN_LED_N); 						//    
	PORTB &= ~_BV(PIN_LED_N); 						//    
	for(j = 0; j < 30000; j++){
		if(!(PINB & _BV(PIN_LED_N))) break; 		//  ,   ,  ,   (0-65000)
	}
	DDRB |= _BV(PIN_LED_N); 						//   
	return j;
}

//  
void led_blink(void){
	PORTB |= _BV(PIN_LED_P); 			
	_delay_ms(1);
	PORTB &= ~_BV(PIN_LED_P);
}

// / 
void motor_work(void){
	for(uint8_t t = 0; t < 10; t++){ 
		led_blink();								//    
		_delay_ms(200); 
	}

	cli();											//     
	PORTB |= _BV(PIN_MOTOR); 
	_delay_ms(MOTOR_WORK_TIME);
	PORTB &= ~_BV(PIN_MOTOR);
	sei();
	
	main_timer = 0;	
}

//----------------------------------------------------------------------------------------------------

//  WDT
void prevent_wdt_reset(void){
	if(MCUSR & _BV(WDRF)){            				//  reset  WDT...
		MCUSR &= ~_BV(WDRF);               		//  WDT reset flag
		WDTCR |= _BV(WDCE) | _BV(WDE);  			//  WD Change Bit
		WDTCR = 0x00;                      			//  WDT
	}
}

//  WDT
void setup_wdt(void){
	WDTCR |= _BV(WDCE) | _BV(WDE);   				//  WD Change Bit
	WDTCR = _BV(WDTIE) |             				//   WDT, _BV(WDIE)  attiny85 
		_BV(WDP3) | _BV(WDP0); 					//    8s _BV(WDP3) | _BV(WDP0); 
}

//     
void mk_sleep_enable(void){
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);    		//   ,  wdt  
	sleep_enable();
	ACSR |= _BV(ACD);                   			//   
	ADCSRA &= ~_BV(ADEN);                 			//  
}

//  WDT
ISR(WDT_vect){
	sleep_flag = false;
}

//    
void button_interrupts_enable(void){
	GIMSK = _BV(PCIE); 							
	PCMSK = _BV(PCINT1) | _BV(PCINT2); 			//    : PCINT1=pb1, PCINT2=pb2
}

//  
ISR(PCINT0_vect){ 
	//_delay_ms(100); 								//    
	if(PINB & _BV(PIN_BUTTON_MODE))  button_mode_flag  = true;
	if(PINB & _BV(PIN_BUTTON_MOTOR)) button_motor_flag = true;
}

//-----------------------------------------------------------------------------------------------------
//										 
//-----------------------------------------------------------------------------------------------------

int main(void){

	// SETUP
	
	uint16_t light_timer = 0; 						//   
	uint8_t mode;									//  	
	uint16_t light_level, light_limit;				//    .  . 0-  , 65000-
	bool light_on_flag = false;						

	DDRB  = 0xff; 									//    ""					
	PORTB = 0x00;									//     ""

	//    .  10 .   
	for(uint8_t t = 0; t < 10; t++){ 
		led_blink();
		_delay_ms(1000); 
	}
	
	light_limit = readLED();						//   . 
	light_limit += 5000; 							//     .
	
	mode = eeprom_read_byte(&Addr);					//     EEPROM
	if(mode == 0xff) mode = DEFAULT_MODE;			//     (0xff),   -	
	
	//    
	cli();
	prevent_wdt_reset(); 						
	setup_wdt(); 								
	mk_sleep_enable();
	button_interrupts_enable();	
	sei();
	
	motor_work();									//   
	
	// LOOP
	
	while(1){
		
		//   MOTOR
		if(button_motor_flag){
			motor_work();
			button_motor_flag = false;
		}	
		
		//   MODE
		if(button_mode_flag){
			cli();									//          EEPROM
			
			mode++;
			if(mode == MODE_COUNT) mode = 0;		//    
			
			eeprom_write_byte(&Addr,mode);			//   EEPROM  
			
			for(uint8_t t = 0; t < mode + 1; t++){
				led_blink();						//       
				_delay_ms(500);
			}
			
			main_timer 	= 0;						//  
			light_timer = 0;
			_delay_ms(2000);
			sei();
			
			button_mode_flag = false;
		}	
		
		//    Watch Dog (WDT)
		if(!sleep_flag){			
			
			main_timer++;
			
			light_level = readLED();				//  	
			
			led_blink();							//  
			
			//     
			if(main_timer == pgm_read_word(&motor_on_time[mode])){
				//   .   .,   :  ( ),      
				if(light_level >= light_limit || light_timer == MAX_LIGHT_TIME) motor_work(); 
					else main_timer = 0;
			}
			
			//  ,     
			//   
			if(light_level < light_limit){
				if(!light_on_flag){ 
					_delay_ms(200); 
					led_blink(); 					//      
				}				
				light_timer++;
				if(light_timer > MAX_LIGHT_TIME) light_timer = MAX_LIGHT_TIME; 	//    
				light_on_flag = true;
			} else {
				//       , 
				if(light_on_flag){
					//            ,     
					if(light_timer > pgm_read_byte(&min_light_time[mode]) && main_timer > MIN_LAST_TIME){ 
						motor_work();
					} else { 
						_delay_ms(200); 
						led_blink(); 				//      
					} 	
					
					light_timer = 0;				//   
					
					light_on_flag = false;
				}
			}
			
			sleep_flag = true;	
				
		} 
		
		sleep_cpu(); 								//  
		
	} 
	
	return 0; 
}

// ========================================================================================================
//
//                                               : 											
//																									
//	1)     , .   ,  ,   ,			
//		     .																		
//	2)   ,  .																
//	3)   :																				
//		1.     (.  2.5 ),        .		
//		2.   ,    .
//			      15 ,   ,     
//			 .												
//	4)      (.  ),   ,  .	
//																											
// ---------------------------------------------------------------------------------------------------------
//
// 								                 :
//
// 1)    ,     - 2.5 
// 2) -//-   , -//- 5 
// 3) -//-  3 , -//- 2.5 
// 4) -//-  3 , -//- 5 
//
// ---------------------------------------------------------------------------------------------------------
//
//								(C)   '2018
//
//						    
//
//						ashelehov@yandex.ru  http://wedfotoart.ru
//
//	  " ",      ,    .
//
// =========================================================================================================
